// Copyright 2024 The Khronos Group
// SPDX-License-Identifier: Apache-2.0

#include "Object.h"

namespace hecore {

// Object definitions /////////////////////////////////////////////////////////

Object::Object(ANARIDataType type, HeCoreDeviceGlobalState *s)
    : helium::BaseObject(type, s)
{}

void Object::commit()
{
  // no-op
}

bool Object::getProperty(
    const std::string_view &name, ANARIDataType type, void *ptr, uint32_t flags)
{
  if (name == "valid" && type == ANARI_BOOL) {
    helium::writeToVoidP(ptr, isValid());
    return true;
  }

  return false;
}

bool Object::isValid() const
{
  return true;
}

HeCoreDeviceGlobalState *Object::deviceState() const
{
  return (HeCoreDeviceGlobalState *)helium::BaseObject::m_state;
}

// UnknownObject definitions //////////////////////////////////////////////////

UnknownObject::UnknownObject(ANARIDataType type, HeCoreDeviceGlobalState *s)
    : Object(type, s)
{}

bool UnknownObject::isValid() const
{
  return false;
}

} // namespace hecore

HECORE_ANARI_TYPEFOR_DEFINITION(hecore::Object *);
